@@ -43,7 +43,6 @@
 #define OUTPUT_KMAIL_DIR_TEMPLATE ".%s.directory"
 #define KMAIL_INDEX ".%s.index"
 
-#define VERSION "0.5.1"
 // max size of the c_time char*. It will store the date of the email
 #define C_TIME_SIZE 500
 #define PERM_DIRS 0777
@@ -144,17 +143,22 @@ int main(int argc, char** argv) {
   int base64_body = 0;
   //  int encrypt = 0;
   FILE *fp;
-  char *enc; // base64 encoded attachment
+  char *enc = NULL; // base64 encoded attachment
   char *boundary = NULL, *b1, *b2; // the boundary marker between multipart sections
   char *temp = NULL; //temporary char pointer
   int attach_num = 0;
+  char *attach_filename = NULL;
   int skip_child = 0;
   struct file_ll  *f, *head;
+  int save_rtf_body = 1;
   prog_name = argv[0];
   // }}}2
-
-  while ((c = getopt(argc, argv, "d:hko:qrSVwc:"))!= -1) {
+  // command-line option handling {{{2
+  while ((c = getopt(argc, argv, "bd:hko:qrSVwc:"))!= -1) {
     switch (c) {
+    case 'b':
+      save_rtf_body = 0;
+      break;
     case 'c':
       if (optarg!=NULL && optarg[0]=='v')
 	contact_mode=CMODE_VCARD;
@@ -200,6 +204,7 @@ int main(int argc, char** argv) {
       break;
     }
   }
+  // }}}2
 
 #ifdef DEBUG_ALL
   // initialize log file
@@ -383,7 +388,7 @@ int main(int argc, char** argv) {
 	  memset(f->name, 0, 10);
 	  //	    sprintf(f->name, "%09i", f->email_count);
 	} else {
-	  f->name = (char*) xmalloc(strlen(item->file_as)+strlen(OUTPUT_TEMPLATE+1));
+	  f->name = (char*) xmalloc(strlen(item->file_as)+strlen(OUTPUT_TEMPLATE)+1);
 	  sprintf(f->name, OUTPUT_TEMPLATE, item->file_as);
 	}
 
@@ -474,12 +479,25 @@ int main(int argc, char** argv) {
 	    // the specification I am following is (hopefully) RFC2426 vCard Mime Directory Profile
 	    fprintf(f->output, "BEGIN:VCARD\n");
 	    fprintf(f->output, "FN:%s\n", rfc2426_escape(item->contact->fullname));
-	    fprintf(f->output, "N:%s;%s;%s;%s;%s\n",
-		    rfc2426_escape((item->contact->surname==NULL?"":item->contact->surname)),
-		    rfc2426_escape((item->contact->first_name==NULL?"":item->contact->first_name)),
-		    rfc2426_escape((item->contact->middle_name==NULL?"":item->contact->middle_name)),
-		    rfc2426_escape((item->contact->display_name_prefix==NULL?"":item->contact->display_name_prefix)),
-		    rfc2426_escape((item->contact->suffix==NULL?"":item->contact->suffix)));
+
+      // Print name in parts...
+	    fprintf(f->output, "N:");
+      if (item->contact->surname != NULL)
+        fprintf(f->output, "%s", rfc2426_escape(item->contact->surname));
+	    fprintf(f->output, ";");
+      if (item->contact->first_name != NULL)
+	      fprintf(f->output, "%s", rfc2426_escape(item->contact->first_name));
+	    fprintf(f->output, ";");
+      if (item->contact->middle_name != NULL)
+	      fprintf(f->output, "%s", rfc2426_escape(item->contact->middle_name));
+	    fprintf(f->output, ";");
+      if (item->contact->display_name_prefix != NULL)
+	      fprintf(f->output, "%s", rfc2426_escape(item->contact->display_name_prefix));
+	    fprintf(f->output, ";");
+      if (item->contact->suffix != NULL)
+	      fprintf(f->output, "%s", rfc2426_escape(item->contact->suffix));
+	    fprintf(f->output, "\n");
+
 	    if (item->contact->nickname != NULL)
 	      fprintf(f->output, "NICKNAME:%s\n", rfc2426_escape(item->contact->nickname));
 	    if (item->contact->address1 != NULL)
@@ -491,37 +509,76 @@ int main(int argc, char** argv) {
 	    if (item->contact->birthday != NULL)
 	      fprintf(f->output, "BDAY:%s\n", rfc2425_datetime_format(item->contact->birthday));
 	    if (item->contact->home_address != NULL) {
-	      fprintf(f->output, "ADR;TYPE=home:%s;%s;%s;%s;%s;%s;%s\n",
-		      rfc2426_escape((item->contact->home_po_box!=NULL?item->contact->home_po_box:"")),
-		      "", // extended Address
-		      rfc2426_escape((item->contact->home_street!=NULL?item->contact->home_street:"")),
-		      rfc2426_escape((item->contact->home_city!=NULL?item->contact->home_city:"")),
-		      rfc2426_escape((item->contact->home_state!=NULL?item->contact->home_state:"")),
-		      rfc2426_escape((item->contact->home_postal_code!=NULL?item->contact->home_postal_code:"")),
-		      rfc2426_escape((item->contact->home_country!=NULL?item->contact->home_country:"")));
+	      fprintf(f->output, "ADR;TYPE=home:");
+        if (item->contact->home_po_box != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->home_po_box));
+	      fprintf(f->output, ";");
+		      // extended Address
+	      fprintf(f->output, ";");
+        if (item->contact->home_street != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->home_street));
+	      fprintf(f->output, ";");
+        if (item->contact->home_city != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->home_city));
+	      fprintf(f->output, ";");
+        if (item->contact->home_state != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->home_state));
+	      fprintf(f->output, ";");
+        if (item->contact->home_postal_code != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->home_postal_code));
+	      fprintf(f->output, ";");
+        if (item->contact->home_country != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->home_country));
+	      fprintf(f->output, "\n");
 	      fprintf(f->output, "LABEL;TYPE=home:%s\n", rfc2426_escape(item->contact->home_address));
 	    }
 	    if (item->contact->business_address != NULL) {
-	      fprintf(f->output, "ADR;TYPE=work:%s;%s;%s;%s;%s;%s;%s\n",
-		      rfc2426_escape((item->contact->business_po_box!=NULL?item->contact->business_po_box:"")),
-		      "", // extended Address
-		      rfc2426_escape((item->contact->business_street!=NULL?item->contact->business_street:"")),
-		      rfc2426_escape((item->contact->business_city!=NULL?item->contact->business_city:"")),
-		      rfc2426_escape((item->contact->business_state!=NULL?item->contact->business_state:"")),
-		      rfc2426_escape((item->contact->business_postal_code!=NULL?item->contact->business_postal_code:"")),
-		      rfc2426_escape((item->contact->business_country!=NULL?item->contact->business_country:"")));
+	      fprintf(f->output, "ADR;TYPE=work:");
+        if (item->contact->business_po_box != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->business_po_box));
+	      fprintf(f->output, ";");
+		      // extended Address
+	      fprintf(f->output, ";");
+        if (item->contact->business_street != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->business_street));
+	      fprintf(f->output, ";");
+        if (item->contact->business_city != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->business_city));
+	      fprintf(f->output, ";");
+        if (item->contact->business_state != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->business_state));
+	      fprintf(f->output, ";");
+        if (item->contact->business_postal_code != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->business_postal_code));
+	      fprintf(f->output, ";");
+        if (item->contact->business_country != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->business_country));
+	      fprintf(f->output, "\n");
 	      fprintf(f->output, "LABEL;TYPE=work:%s\n", rfc2426_escape(item->contact->business_address));
 	    }
 	    if (item->contact->other_address != NULL) {
-	      fprintf(f->output, "ADR;TYPE=postal:%s;%s;%s;%s;%s;%s;%s\n",
-		      rfc2426_escape((item->contact->other_po_box!=NULL?item->contact->business_po_box:"")),
-		      "", // extended Address
-		      rfc2426_escape((item->contact->other_street!=NULL?item->contact->other_street:"")),
-		      rfc2426_escape((item->contact->other_city!=NULL?item->contact->other_city:"")),
-		      rfc2426_escape((item->contact->other_state!=NULL?item->contact->other_state:"")),
-		      rfc2426_escape((item->contact->other_postal_code!=NULL?item->contact->other_postal_code:"")),
-		      rfc2426_escape((item->contact->other_country!=NULL?item->contact->other_country:"")));
-	      fprintf(f->output, "ADR;TYPE=postal:%s\n", rfc2426_escape(item->contact->other_address));
+	      fprintf(f->output, "ADR;TYPE=postal:");
+        if (item->contact->other_po_box != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->other_po_box));
+	      fprintf(f->output, ";");
+		      // extended Address
+	      fprintf(f->output, ";");
+        if (item->contact->other_street != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->other_street));
+	      fprintf(f->output, ";");
+        if (item->contact->other_city != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->other_city));
+	      fprintf(f->output, ";");
+        if (item->contact->other_state != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->other_state));
+	      fprintf(f->output, ";");
+        if (item->contact->other_postal_code != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->other_postal_code));
+	      fprintf(f->output, ";");
+        if (item->contact->other_country != NULL)
+		      fprintf(f->output, "%s", rfc2426_escape(item->contact->other_country));
+	      fprintf(f->output, "\n");
+	      fprintf(f->output, "LABEL;TYPE=postal:%s\n", rfc2426_escape(item->contact->other_address));
 	    }
 	    if (item->contact->business_fax != NULL)
 	      fprintf(f->output, "TEL;TYPE=work,fax:%s\n", rfc2426_escape(item->contact->business_fax));
@@ -558,12 +615,12 @@ int main(int argc, char** argv) {
 	    if (item->contact->profession != NULL)
 	      fprintf(f->output, "ROLE:%s\n", rfc2426_escape(item->contact->profession));
 	    if (item->contact->assistant_name != NULL || item->contact->assistant_phone != NULL) {
-	      fprintf(f->output, "AGENT:BEGIN:VCARD\\n");
+	      fprintf(f->output, "AGENT:BEGIN:VCARD\n");
 	      if (item->contact->assistant_name != NULL)
-		fprintf(f->output, "FN:%s\\n", rfc2426_escape(item->contact->assistant_name));
+		fprintf(f->output, "FN:%s\n", rfc2426_escape(item->contact->assistant_name));
 	      if (item->contact->assistant_phone != NULL)
-		fprintf(f->output, "TEL:%s\\n", rfc2426_escape(item->contact->assistant_phone));
-	      fprintf(f->output, "END:VCARD\\n\n");
+		fprintf(f->output, "TEL:%s\n", rfc2426_escape(item->contact->assistant_phone));
+	      fprintf(f->output, "END:VCARD\n\n");
 	    }
 	    if (item->contact->company_name != NULL)
 	      fprintf(f->output, "ORG:%s\n", rfc2426_escape(item->contact->company_name));
@@ -678,29 +735,30 @@ int main(int argc, char** argv) {
 	  DEBUG_EMAIL(("item->email->subject->subj = %p\n", item->email->subject->subj));
 	}
 	if (item->email->header != NULL) {
+	  char *soh = NULL;  // real start of headers.
+
 	  // some of the headers we get from the file are not properly defined.
 	  // they can contain some email stuff too. We will cut off the header
 	  // when we see a \n\n or \r\n\r\n
-
 	  removeCR(item->email->header);
-
 	  temp = strstr(item->email->header, "\n\n");
-
 	  if (temp != NULL) {
 	    DEBUG_MAIN(("main: Found body text in header\n"));
-	    temp += 2; // get past the \n\n
-	    *temp = '\0';
-	  }
+	    temp[1] = '\0'; // stop after first \n
+    }
 	  
+    // Now, write out the header...
+	  soh = skip_header_prologue(item->email->header);
 	  if (mode != MODE_SEPERATE) {
-	    char *soh = NULL;  // real start of headers.
 	    // don't put rubbish in if we are doing seperate
-	    fprintf(f->output, "From \"%s\" %s\n", item->email->outlook_sender_name, c_time);
-	    soh = skip_header_prologue(item->email->header);
-	    fprintf(f->output, "%s\n\n", soh);
-	  } else {
-	    fprintf(f->output, "%s\n", item->email->header);
-	  }
+      if ( strncmp(soh, "X-From_: ", 9) == 0 ) {
+        fputs("From ", f->output);
+        soh += 9;
+      } else
+	      fprintf(f->output, "From \"%s\" %s\n", item->email->outlook_sender_name, c_time);
+    }
+	  fprintf(f->output, "%s", soh);
+
 	} else {
 	  //make up our own header!
 	  if (mode != MODE_SEPERATE) {
@@ -730,31 +788,31 @@ int main(int argc, char** argv) {
 	    fprintf(f->output, "Date: %s\n", c_time);
 	    free(c_time);
 	  }
+	}
 
 	  fprintf(f->output, "MIME-Version: 1.0\n");
 	  if (item->attach != NULL) {
 	    // write the boundary stuff if we have attachments
 	    fprintf(f->output, "Content-type: multipart/mixed;\n\tboundary=\"%s\"\n",
 		    boundary);
-	  } else if (item->email->htmlbody && item->email->body) {
-	    // else if we have an html and text body then tell it so
-	    fprintf(f->output, "Content-type: multipart/alternate;\n\tboundary=\"%s\"\n",
+	  } else if (boundary) {
+	    fprintf(f->output, "Content-type: multipart/alternative;\n\tboundary=\"%s\"\n",
 		    boundary);
 	  } else if (item->email->htmlbody) {
 	    fprintf(f->output, "Content-type: text/html\n");
 	  }
 	  fprintf(f->output, "\n");
-	}
 	
-
 	DEBUG_MAIN(("main: About to print Body\n"));
+	fprintf(f->output, "\n\n");	// separate headers from body
 
 	if (item->email->body != NULL) {
 	  if (boundary) {
 	    fprintf(f->output, "\n--%s\n", boundary);
-	    fprintf(f->output, "Content-type: text/plain\n\n");
+	    fprintf(f->output, "Content-type: text/plain\n");
 	    if (base64_body)
 	      fprintf(f->output, "Content-Transfer-Encoding: base64\n");
+	    fprintf(f->output, "\n");
 	  }
 	  removeCR(item->email->body);
 	  if (base64_body)
@@ -767,9 +825,10 @@ int main(int argc, char** argv) {
 	if (item->email->htmlbody != NULL) {
 	  if (boundary) {
 	    fprintf(f->output, "\n--%s\n", boundary);
-	    fprintf(f->output, "Content-type: text/html\n\n");
+	    fprintf(f->output, "Content-type: text/html\n");
 	    if (base64_body)
 	      fprintf(f->output, "Content-Transfer-Encoding: base64\n");
+	    fprintf(f->output, "\n");
 	  }
 	  removeCR(item->email->htmlbody);
 	  if (base64_body)
@@ -781,7 +840,7 @@ int main(int argc, char** argv) {
 
 	attach_num = 0;
 
-	if (item->email->rtf_compressed != NULL) {
+	if ( item->email->rtf_compressed != NULL && save_rtf_body ) {
 	  DEBUG_MAIN(("Adding RTF body as attachment\n"));
 	  item->current_attach = (pst_item_attach*)xmalloc(sizeof(pst_item_attach));
 	  memset(item->current_attach, 0, sizeof(pst_item_attach));
@@ -833,18 +892,27 @@ int main(int argc, char** argv) {
 	  }
 	  if (mode == MODE_SEPERATE) {
 	    f->name = check_filename(f->name);
-	    if (item->current_attach->filename2 == NULL) {
+      // If there is a long filename (filename2) use that, otherwise
+      // use the 8.3 filename (filename1)
+      if (item->current_attach->filename2) {
+        attach_filename = item->current_attach->filename2;
+      } else {
+        attach_filename = item->current_attach->filename1;
+      }
+	    if (attach_filename == NULL) {
+        // generate our own (dummy) filename for the attachement
 	      temp = xmalloc(strlen(f->name)+15);
 	      sprintf(temp, "%s-attach%i", f->name, attach_num);
 	    } else {
-	      temp = xmalloc(strlen(f->name)+strlen(item->current_attach->filename2)+15);
+        // have an attachment name, make sure it's unique
+	      temp = xmalloc(strlen(f->name)+strlen(attach_filename)+15);
 	      fp = NULL; x=0;
 	      do {
 		if (fp != NULL) fclose(fp);
 		if (x == 0)
-		  sprintf(temp, "%s-%s", f->name, item->current_attach->filename2);
+		  sprintf(temp, "%s-%s", f->name, attach_filename);
 		else
-		  sprintf(temp, "%s-%s-%i", f->name, item->current_attach->filename2, x);
+		  sprintf(temp, "%s-%s-%i", f->name, attach_filename, x);
 	      } while ((fp = fopen(temp, "r"))!=NULL && ++x < 99999999);
 	      if (x > 99999999) {
 	        DIE(("error finding attachment name. exhausted possibilities to %s\n", temp));
@@ -861,6 +929,7 @@ int main(int argc, char** argv) {
 	      }
 	      fclose(fp);
 	    }
+      if (temp) free(temp); temp = NULL;
 	  } else {
 	    DEBUG_MAIN(("main: Attachment Size is %i\n", item->current_attach->size));
 	    DEBUG_MAIN(("main: Attachment Pointer is %p\n", item->current_attach->data));
@@ -879,11 +948,18 @@ int main(int argc, char** argv) {
 		fprintf(f->output, "Content-type: %s\n", item->current_attach->mimetype);
 	      }
 	      fprintf(f->output, "Content-transfer-encoding: base64\n");
-	      if (item->current_attach->filename2 == NULL) {
+        // If there is a long filename (filename2) use that, otherwise
+        // use the 8.3 filename (filename1)
+        if (item->current_attach->filename2) {
+          attach_filename = item->current_attach->filename2;
+        } else {
+          attach_filename = item->current_attach->filename1;
+        }
+	      if (attach_filename == NULL) {
 		fprintf(f->output, "Content-Disposition: inline\n\n");
 	      } else {
 		fprintf(f->output, "Content-Disposition: attachment; filename=\"%s\"\n\n",
-			item->current_attach->filename2);
+			attach_filename);
 	      }
 	    }
 	    if (item->current_attach->data != NULL) {
@@ -1111,14 +1187,14 @@ char *removeCR (char *c) {
   // converts /r/n to /n
   char *a, *b;
   DEBUG_ENT("removeCR");
-  a = b = c;
-  while (*a != '\0') {
+
+  for (a = b = c; *a != '\0'; ++a, ++b) {
+    if (*a == '\r')
+      a++;
     *b = *a;
-    if (*a != '\r')
-      b++;
-    a++;
   }
   *b = '\0';
+
   DEBUG_RET();
   return c;
 }
@@ -1129,6 +1205,7 @@ int usage() {
   version();
   printf("Usage: %s [OPTIONS] {PST FILENAME}\n", prog_name);
   printf("OPTIONS:\n");
+  printf("\t-b\t- Don't save RTF-Body attachments\n");
   printf("\t-c[v|l]\t- Set the Contact output mode. -cv = VCard, -cl = EMail list\n");
   printf("\t-d\t- Debug to file. This is a binary log. Use readlog to print it\n");
   printf("\t-h\t- Help. This screen\n");
@@ -1146,7 +1223,7 @@ int usage() {
 // int version() {{{1
 int version() {
   DEBUG_ENT("version");
-  printf("ReadPST v%s implementing LibPST v%s\n", VERSION, PST_VERSION);
+  printf("ReadPST v%s.\n", VERSION);
 #if BYTE_ORDER == BIG_ENDIAN
   printf("Big Endian implementation being used.\n");
 #elif BYTE_ORDER == LITTLE_ENDIAN
@@ -1396,47 +1473,62 @@ char *check_filename(char *fname) {
 // }}}1
 // char *rfc2426_escape(char *str) {{{1
 char *rfc2426_escape(char *str) {
-  static char* buf = NULL;
-  char *ret, *a, *b;
-  int x = 0, y, z;
+  static char *buf = NULL;
+  static int buflen = 0;
+  char *a, *b;
+  int x, y, z;
   DEBUG_ENT("rfc2426_escape");
-  if (str == NULL) 
-    ret = str;
-  else {
-    
-    // calculate space required to escape all the following characters
-    x = strlen(str) +(y=(chr_count(str, ',')*2) + (chr_count(str, '\\')*2) + (chr_count(str, ';')*2) + (chr_count(str, '\n')*2));
-    z = chr_count(str, '\r');
-    if (y == 0 && z == 0)
-      // there isn't any extra space required
-      ret = str;
-    else {
-      buf = (char*) realloc(buf, x+1);
-      a = str;
-      b = buf;
-      while (*a != '\0') {
-	switch(*a) {
-	case ',' :
-	case '\\':
-	case ';' :
-	case '\n':
-	  *(b++)='\\';
-	  *b=*a;
-	break;
-	case '\r':
-	  break;
-	default:
-	  *b=*a;
-	}
-	b++;
-      a++;
-      }
-      *b = '\0';
-      ret = buf;
-    }
+
+	if (str == NULL) {
+		DEBUG_RET();
+		return NULL;
   }
+    
+  // calculate space required to escape all the following characters
+  y = chr_count(str, ',')
+    + chr_count(str, '\\')
+    + chr_count(str, ';')
+    + chr_count(str, '\n');
+  z = chr_count(str, '\r');
+  x = strlen(str) + y - z + 1; // don't forget room for the NUL
+
+	if (y == 0 && z == 0) {
+		// there isn't any work required
+		DEBUG_RET();
+		return str;
+	}
+
+  if ( x > buflen )  // if we need a bigger buffer than we have...
+  {
+    buflen = x;
+    buf = (char*) realloc(buf, buflen);
+	  if ( buf == NULL )
+	  {
+        fprintf(stderr, "Error: rfc2426_escape(): realloc(%d) returned NULL!\n", x + 1);
+        exit(1);
+	  }
+  }
+
+  for ( a = str, b = buf; *a != '\0'; ++a, ++b )
+	  switch(*a) {
+      case ',' :
+      case '\\':
+      case ';' :
+	      *b++ = '\\', *b = *a;	// escape with a '\\'
+	      break;
+      case '\n':	// newlines are encoded as "\n"
+	      *b++ = '\\', *b = 'n';
+	      break;
+      case '\r': // skip carriage returns
+        --b;
+	      break;
+      default:
+	      *b = *a;
+	  }
+  *b = '\0'; // NUL-terminate the string (buf)
+
   DEBUG_RET();
-  return ret;
+  return buf;
 }
 // }}}1
 // int chr_count(char *str, char x) {{{1
@@ -1501,5 +1593,5 @@ char *skip_header_prologue(char *headers) {
 }
 // }}}1
 
-// vim:sw=4 ts=4:
+// vim:sw=2 ts=2 et:
 // vim600: set foldlevel=0 foldmethod=marker:
